<?php

namespace Drupal\phpunit_tests\Form;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\phpunit_tests\PhpunitTestsResourceService;
use Drupal\testsuite\BaseTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the database logging filter form.
 *
 * @internal
 */
class PhpunitTestsFilterForm extends FormBase {
  use BaseTrait;

  /**
   * The array of filters.
   *
   * @var array
   */
  protected $filters = [
    'action',
    'area',
    'module',
    'directory',
    'test',
  ];

  /**
   * The database.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Load the Resource Service.
   *
   * @var \Drupal\phpunit_tests\PhpunitTestsResourceService
   */
  protected $phpunitTestsResourceService;

  /**
   * The time interface.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $timeInterface;

  /**
   * Load Config Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * PhpUnitTestsForm constructor.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The database.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   Load messenger service.
   * @param \Drupal\phpunit_tests\PhpunitTestsResourceService $phpunitTestsResourceService
   *   The load resource service.
   * @param \Drupal\Component\Datetime\TimeInterface $timeInterface
   *   The load resource service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config service.
   */
  public function __construct(
    Connection $connection,
    MessengerInterface $messenger,
    PhpunitTestsResourceService $phpunitTestsResourceService,
    TimeInterface $timeInterface,
    ConfigFactoryInterface $configFactory
  ) {
    $this->connection = $connection;
    $this->messenger = $messenger;
    $this->phpunitTestsResourceService = $phpunitTestsResourceService;
    $this->timeInterface = $timeInterface;
    $this->configFactory = $configFactory;
    ini_set('max_execution_time', 0);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('messenger'),
      $container->get('phpunit_tests.load_resource.service'),
      $container->get('datetime.time'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'phpunit_tests_filter_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $form['filters'] = [
      '#type' => 'details',
      '#title' => $this->t('Filter log messages'),
      '#open' => TRUE,
      '#attached' => [
        'library' => [
          'phpunit_tests/phpunit_tests.filters',
        ],
      ],
    ];

    $session_filters = $this->getRequest()->getSession()->get('phpunit_tests_overview_filter', []);

    $form['filters']['filters-container'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'phpunit-tests-filter-form-wrapper',
      ],
    ];

    $action_dropdown = $form_state->hasValue('action') ? $form_state->getValue('action') : '';

    $form['filters']['filters-container']['action-container'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'phpunit-test-action-container',
      ],
    ];
    $form['filters']['filters-container']['action-container']['action'] = [
      '#title' => 'Actions',
      '#type' => 'select',
      '#multiple' => FALSE,
      '#size' => 8,
      '#options' => [
        'filter' => $this->t('Filter'),
        'test' => $this->t('Run Test'),
      ],
      '#default_value' => !empty($session_filters['action']) ? $session_filters['action'] : $action_dropdown,
    ];

    $testDirectoryDropdown = $form_state->hasValue('directory') ? $form_state->getValue('directory') : '';
    $test_directory_dropdown = !empty($session_filters['directory']) ? $session_filters['directory'] : $testDirectoryDropdown;

    $form['filters']['filters-container']['test-directory-container'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'phpunit-test-directories-container',
      ],
    ];
    $form['filters']['filters-container']['test-directory-container']['directory'] = [
      '#title' => 'Test Directory',
      '#type' => 'select',
      '#multiple' => FALSE,
      '#size' => 8,
      '#options' => $this->getEnabledTestDirectories(),
      '#default_value' => $test_directory_dropdown,
    ];

    $areaDropdown = $form_state->hasValue('area') ? $form_state->getValue('area') : '';
    $area_dropdown = !empty($session_filters['area']) ? $session_filters['area'] : $areaDropdown;

    $form['filters']['filters-container']['area-container'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'phpunit-test-areas-container',
      ],
    ];
    $form['filters']['filters-container']['area-container']['area'] = [
      '#title' => 'Area',
      '#type' => 'select',
      '#multiple' => FALSE,
      '#size' => 8,
      '#options' => [
        'core' => $this->t('Core'),
        'contrib' => $this->t('Contrib'),
        'custom' => $this->t('Custom'),
      ],
      '#default_value' => $area_dropdown,
      '#ajax' => [
        'callback' => [$this, 'getModulesCallback'],
        'event' => 'change',
        'method' => 'replace',
        'wrapper' => 'phpunit-test-modules-container',
        'progress' => ['type' => 'none'],
      ],
    ];

    $moduleDropdown = $form_state->hasValue('module') ? $form_state->getValue('module') : '';
    $module_dropdown = !empty($session_filters['module']) ? $session_filters['module'] : $moduleDropdown;

    $form['filters']['filters-container']['module-container'] = [
      '#type' => 'container',
      '#states' => [
        'visible' => [
          ':input[name="area"]' => [
            [
              '!value' => '',
            ],
          ],
        ],
      ],
    ];
    $form['filters']['filters-container']['module-container']['module'] = [
      '#title' => 'Module',
      '#type' => 'select',
      '#multiple' => FALSE,
      '#size' => 8,
      '#options' => $this->phpunitTestsResourceService->getResource('module', $area_dropdown, 'all', $test_directory_dropdown),
      '#default_value' => $module_dropdown,
      '#prefix' => '<div id="phpunit-test-modules-container">',
      '#suffix' => '</div>',
      '#states' => [
        'visible' => [
          ':input[name="area"]' => [
            [
              '!value' => '',
            ],
          ],
        ],
      ],
      '#ajax' => [
        'callback' => [$this, 'getTestsCallback'],
        'event' => 'change',
        'method' => 'replace',
        'wrapper' => 'phpunit-tests-container',
        'progress' => ['type' => 'none'],
      ],
    ];

    $testDropdown = $form_state->hasValue('test') ? $form_state->getValue('test') : '';
    $test_dropdown = !empty($session_filters['test']) ? $session_filters['test'] : $testDropdown;

    $form['filters']['filters-container']['tests-container'] = [
      '#type' => 'container',
      '#states' => [
        'visible' => [
          ':input[name="module"]' => [
            [
              '!value' => '',
            ],
          ],
        ],
      ],
    ];
    $form['filters']['filters-container']['tests-container']['test'] = [
      '#title' => 'Tests',
      '#type' => 'select',
      '#multiple' => FALSE,
      '#size' => 8,
      '#options' => $this->phpunitTestsResourceService->getResource('test', $area_dropdown, $module_dropdown, $test_directory_dropdown),
      '#default_value' => $test_dropdown,
      '#prefix' => '<div id="phpunit-tests-container">',
      '#suffix' => '</div>',
      '#states' => [
        'visible' => [
          ':input[name="module"]' => [
            [
              '!value' => '',
            ],
          ],
        ],
      ],
    ];

    $form['filters']['actions'] = [
      '#type' => 'actions',
      '#attributes' => ['class' => ['container-inline']],
    ];
    $form['filters']['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];
    if (!empty($session_filters)) {
      $form['filters']['actions']['reset'] = [
        '#type' => 'submit',
        '#value' => $this->t('Reset'),
        '#limit_validation_errors' => [],
        '#submit' => ['::resetForm'],
      ];
    }
    return $form;
  }

  /**
   * Selects the second dropdown to be returned for re-rendering.
   *
   * @return array
   *   The renderable element.
   */
  public function getModulesCallback($form, $form_state) {
    return $form['filters']['filters-container']['module-container'];
  }

  /**
   * Selects the fourth dropdown to be returned for re-rendering.
   *
   * @return array
   *   The renderable element.
   */
  public function getTestsCallback($form, $form_state) {
    return $form['filters']['filters-container']['tests-container'];
  }

  /**
   * Gets an array of enabled test folders.
   *
   * @return array
   *   The array of enabled test folders.
   */
  private function getEnabledTestDirectories() {
    $enabledTests = [];
    $config = $this->configFactory->get('phpunit_tests.settings');
    foreach ($config->get('testsuite_loaded_test_types') as $test => $enabled) {
      if ($enabled != 0) {
        $enabledTests[$test] = $test;
      }
    }
    return $enabledTests;
  }

  /**
   * Creates a log entry.
   *
   * @param string $area
   *   The area the module is located. Core, contrib or custom.
   * @param string $module
   *   The module name.
   * @param string $directory
   *   The test directory. Unit, Functional etc.
   * @param string $test
   *   The absolute path the tests.
   * @param array $data
   *   The area where the module is. Core, contrib or custom.
   *
   * @return bool
   *   True or false depending on if insert succeeds.
   */
  private function createLog($area, $module, $directory, $test, $data) {
    $db1 = $this->connection->insert('phpunit_test')
      ->fields([
        'created' => $this->timeInterface->getRequestTime(),
      ])
      ->execute();

    $db2 = $this->connection->insert('phpunit_test_item')
      ->fields([
        'phpunit_test_id' => $db1,
        'area' => $area,
        'module' => $module,
        'directory' => $directory,
        'test' => $test,
        'error_report' => $data,
      ])
      ->execute();

    if ($db1 && $db2) {
      return TRUE;
    }
    else {
      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if ($form_state->isValueEmpty('action')) {
      $form_state->setErrorByName('action', $this->t('You must select something to filter by.'));
    }
    if (!in_array($form_state->getValue('action'), ['filter', 'test'])) {
      $form_state->setErrorByName('action', $this->t('Invalid option.'));
    }
    if ($form_state->isValueEmpty('area')) {
      $form_state->setErrorByName('area', $this->t('You must select something to filter by.'));
    }
    if (!in_array($form_state->getValue('area'), $this->areas)) {
      $form_state->setErrorByName('area', $this->t('Invalid option.'));
    }
    if ($form_state->isValueEmpty('module')) {
      $form_state->setErrorByName('module', $this->t('You must select something to filter by.'));
    }
    if (!preg_match('/[A-za-z\s]+/', $form_state->getValue('module'))) {
      $form_state->setErrorByName('area', $this->t('Invalid option.'));
    }
    if ($form_state->isValueEmpty('directory')) {
      $form_state->setErrorByName('directory', $this->t('You must select something to filter by.'));
    }
    if (!in_array($form_state->getValue('directory'), $this->directories)) {
      $form_state->setErrorByName('directory', $this->t('Invalid option.'));
    }
    if (!preg_match('/[A-Za-z]+|/', $form_state->getValue('test'))) {
      $form_state->setErrorByName('test', $this->t('Invalid option.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $session_filters = $this->getRequest()->getSession()->get('phpunit_tests_overview_filter', []);

    if ($form_state->getValue('action') == 'test') {

      if ($form_state->isValueEmpty('test')) {
        $results = $this->phpunitTestsResourceService->getPhpunitStatement(
          $form_state->getValue('area'),
          $form_state->getValue('module'),
          $form_state->getValue('directory')
        );
      }
      else {
        $results = $this->phpunitTestsResourceService->getPhpunitStatement(
          $form_state->getValue('area'),
          $form_state->getValue('module'),
          $form_state->getValue('directory'),
          $form_state->getValue('test')
              );
      }

      if ($results != NULL) {
        if ($this->createLog(
          $form_state->getValue('area'),
          $form_state->getValue('module'),
          $form_state->getValue('directory'),
          $form_state->getValue('test'),
          $results
        )) {
          $this->messenger->addStatus('A phpunit test has been recorded.');
        }
        else {
          $this->messenger->addStatus('A phpunit test could not not been recorded. Please be sure all packages are loaded and the phpunit.xml is created with the right data.');
        }
      }
      else {
        $this->messenger->addStatus('A phpunit test could not not been recorded. Please be sure all packages are loaded and the phpunit.xml is created with the right data.');
      }
    }

    foreach ($this->filters as $name) {
      if ($form_state->hasValue($name)) {
        $session_filters[$name] = $form_state->getValue($name);
      }
    }
    $this->getRequest()->getSession()->set('phpunit_tests_overview_filter', $session_filters);
    $this->messenger->addStatus('Filters updated.');
  }

  /**
   * Resets the filter form.
   */
  public function resetForm() {
    $this->getRequest()->getSession()->remove('phpunit_tests_overview_filter');
  }

}
